"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.fromDirectRepresentation = fromDirectRepresentation;
exports.fromOnesComplementRepresentation = fromOnesComplementRepresentation;
exports.fromTwosComplementRepresentation = fromTwosComplementRepresentation;
exports.toDirectRepresentation = toDirectRepresentation;
exports.toOnesComplementRepresentation = toOnesComplementRepresentation;
exports.toTwosComplementRepresentation = toTwosComplementRepresentation;

var _isSafeInteger = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/number/is-safe-integer"));

var _bignumber = _interopRequireDefault(require("bignumber.js"));

var _converters = require("./converters");

/** @param {number} bitsCount */
function checkBitsCount(bitsCount) {
  if (!(0, _isSafeInteger["default"])(bitsCount)) throw new Error('bits count is not a safe integer');
  if (bitsCount <= 0) throw new Error('bits count is not positive');
}
/**
 * @param {number | BigNumber} number
 * @param {number} bitsCount
 * @returns {BigNumber}
 */


function toDirectRepresentation(number, bitsCount) {
  checkBitsCount(bitsCount);
  number = (0, _converters.toBigInteger)(number);
  var abs = number.abs();
  if (abs.gte((0, _converters.toTwosPower)(bitsCount - 1))) throw new Error("int".concat(bitsCount, " overflow"));
  return number.isNegative() ? abs.plus((0, _converters.toTwosPower)(bitsCount - 1)) : abs;
}
/**
 * @param {BigNumber} representation
 * @param {number} bitsCount
 */


function checkRepresentation(representation, bitsCount) {
  checkBitsCount(bitsCount);
  if (!_bignumber["default"].isBigNumber(representation)) throw new Error('representation is not a BigNumber');
  if (representation.isNegative()) throw new Error('representation is negative');

  if (representation.gte((0, _converters.toTwosPower)(bitsCount))) {
    throw new Error("".concat(bitsCount, "-bit representation overflow"));
  }
}
/**
 * @param {BigNumber} directRepresentation
 * @param {number} bitsCount
 * @returns {BigNumber}
 */


function fromDirectRepresentation(directRepresentation, bitsCount) {
  checkRepresentation(directRepresentation, bitsCount);
  var postMaxValue = (0, _converters.toTwosPower)(bitsCount - 1);
  var isNegative = directRepresentation.gte(postMaxValue);
  var abs = isNegative ? directRepresentation.minus(postMaxValue) : directRepresentation;
  return isNegative ? abs.times(-1) : abs;
}
/**
 * @param {number | BigNumber} number
 * @param {number} bitsCount
 * @returns {BigNumber}
 */


function toOnesComplementRepresentation(number, bitsCount) {
  number = (0, _converters.toBigInteger)(number);
  var directRepresentation = toDirectRepresentation(number, bitsCount);
  if (!number.isNegative()) return directRepresentation;
  return (0, _converters.toTwosPower)(bitsCount - 1).minus(1).plus((0, _converters.toTwosPower)(bitsCount)).minus(directRepresentation);
}
/**
 * @param {BigNumber} onesComplementRepresentation
 * @param {number} bitsCount
 * @returns {BigNumber}
 */


function fromOnesComplementRepresentation(onesComplementRepresentation, bitsCount) {
  checkRepresentation(onesComplementRepresentation, bitsCount);
  var isNegative = onesComplementRepresentation.gte((0, _converters.toTwosPower)(bitsCount - 1));
  return isNegative ? fromDirectRepresentation(new _bignumber["default"](2).pow(bitsCount - 1).minus(1).plus((0, _converters.toTwosPower)(bitsCount)).minus(onesComplementRepresentation), bitsCount) : fromDirectRepresentation(onesComplementRepresentation, bitsCount);
}
/**
 * @param {number | BigNumber} number
 * @param {number} bitsCount
 * @returns {BigNumber}
 */


function toTwosComplementRepresentation(number, bitsCount) {
  number = (0, _converters.toBigInteger)(number);
  var onesComplementRepresentation = toOnesComplementRepresentation(number, bitsCount);
  if (!number.isNegative()) return onesComplementRepresentation;
  return onesComplementRepresentation.plus(1);
}
/**
 * @param twosComplementRepresentation
 * @param bitsCount
 * @returns {BigNumber}
 */


function fromTwosComplementRepresentation(twosComplementRepresentation, bitsCount) {
  checkRepresentation(twosComplementRepresentation, bitsCount);
  var isNegative = twosComplementRepresentation.gte((0, _converters.toTwosPower)(bitsCount - 1));
  return fromOnesComplementRepresentation(isNegative ? twosComplementRepresentation.minus(1) : twosComplementRepresentation, bitsCount);
}