"use strict";

var _sliceInstanceProperty = require("@babel/runtime-corejs3/core-js-stable/instance/slice");

var _Array$from = require("@babel/runtime-corejs3/core-js-stable/array/from");

var _Symbol = require("@babel/runtime-corejs3/core-js-stable/symbol");

var _getIteratorMethod = require("@babel/runtime-corejs3/core-js/get-iterator-method");

var _Array$isArray2 = require("@babel/runtime-corejs3/core-js-stable/array/is-array");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.checkAbiFormat = checkAbiFormat;
exports.checkBytesCount = checkBytesCount;
exports.checkIntegerSize = checkIntegerSize;
exports.getMethodHash = getMethodHash;
exports.getSignature = getSignature;

var _typeof2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/typeof"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _isSafeInteger = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/number/is-safe-integer"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _assert = require("assert");

var _jsSha = require("js-sha3");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof _Symbol !== "undefined" && _getIteratorMethod(o) || o["@@iterator"]; if (!it) { if (_Array$isArray2(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { var _context3; if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = _sliceInstanceProperty(_context3 = Object.prototype.toString.call(o)).call(_context3, 8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return _Array$from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/** @typedef {import("../../../types/interfaces/Abi").AbiMethod} AbiMethod */

/**
 * @param {AbiMethod} abiMethod
 * @returns {string}
 */
function getSignature(abiMethod) {
  var _context, _context2;

  return (0, _concat["default"])(_context = "".concat(abiMethod.name, "(")).call(_context, (0, _map["default"])(_context2 = abiMethod.inputs).call(_context2, function (_ref) {
    var type = _ref.type;
    return type;
  }).join(','), ")");
}

function checkIntegerSize(bitsCount) {
  if (typeof bitsCount !== 'number') throw new Error('bits count is not a number');
  if (bitsCount <= 0) throw new Error('bits count is not positive');
  if (bitsCount > 256) throw new Error('bits count is greater than 256');
  if (!(0, _isSafeInteger["default"])(bitsCount)) throw new Error('bits count is not a integer');
  if (bitsCount % 8 !== 0) throw new Error('bits count is not divisible to 8');
}

function checkBytesCount(bytesCount) {
  if (typeof bytesCount !== 'number') throw new Error('bytes count is not a number');
  if (bytesCount <= 0) throw new Error('bytes count is not positive');
  if (!(0, _isSafeInteger["default"])(bytesCount)) throw new Error('bytes count is not a integer');
  if (bytesCount > 32) throw new Error('bytes count is grater than 32');
}
/**
 * @param {AbiMethod} abiMethod
 * @returns {string}
 */


function getMethodHash(abiMethod) {
  return (0, _jsSha.keccak256)(getSignature(abiMethod));
}
/** @param {Array<AbiMethod>} abi */


function checkAbiFormat(abi) {
  if (!(0, _isArray["default"])(abi)) throw new Error('abi is not an array');

  var _iterator = _createForOfIteratorHelper(abi),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var abiMethod = _step.value;
      // typeof is uncovered. see https://github.com/gotwarlost/istanbul/issues/582
      if ((0, _typeof2["default"])(abiMethod) !== 'object' || abiMethod === null) throw new Error('abi method is not an object');

      if (abiMethod.type === 'fallback' || abiMethod.type === 'receive') {
        (0, _assert.ok)(abiMethod.payable === true || abiMethod.stateMutability === 'payable', 'abi method is not payable');
        (0, _assert.ok)(abiMethod.inputs === undefined, 'unexpected inputs array of fallback-function');
        (0, _assert.ok)(abiMethod.outputs === undefined, 'unexpected outputs array of fallback-function');
      } else {
        if (abiMethod.type !== 'constructor' && typeof abiMethod.name !== 'string') {
          throw new Error('abi method name is not a string');
        }

        if (!(0, _isArray["default"])(abiMethod.inputs)) throw new Error('inputs of abi method is not an array');

        if (abiMethod.type === 'function' && !(0, _isArray["default"])(abiMethod.outputs)) {
          throw new Error('outputs of abi method is not an array');
        }
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}