"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _fill = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/fill"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _ecurve = require("ecurve");

var _bigi = _interopRequireDefault(require("bigi"));

var _bs = require("bs58");

var _deepEqual = _interopRequireDefault(require("deep-equal"));

var _assert = _interopRequireDefault(require("assert"));

var _hash = require("../hash");

var _publicKey = _interopRequireDefault(require("./public-key"));

var secp256k1 = (0, _ecurve.getCurveByName)('secp256k1');
var n = secp256k1.n;

var toPublic = function toPublic(data) {
  return !data || data.Q ? data : _publicKey["default"].fromStringOrThrow(data);
};

var PrivateKey = /*#__PURE__*/function () {
  /**
   *  @constructor
   *
   *  @param {BigInteger} d
   */
  function PrivateKey(d) {
    (0, _classCallCheck2["default"])(this, PrivateKey);
    this.d = d;
  }
  /**
   *  @method fromBuffer
   *
   *  @param  {Buffer} buf
   *
   *  @return {PrivateKey}
   */


  (0, _createClass2["default"])(PrivateKey, [{
    key: "toWif",
    value:
    /**
     *  @method toWif
     *
     *  @return {String}
     */
    function toWif() {
      var privateKey = this.toBuffer(); // checksum includes the version

      privateKey = (0, _concat["default"])(Buffer).call(Buffer, [Buffer.from([0x80]), privateKey]);
      var checksum = (0, _hash.sha256)(privateKey);
      checksum = (0, _hash.sha256)(checksum);
      checksum = (0, _slice["default"])(checksum).call(checksum, 0, 4);
      var privateWIF = (0, _concat["default"])(Buffer).call(Buffer, [privateKey, checksum]);
      return (0, _bs.encode)(privateWIF);
    }
    /**
     *  @method toWif
     *
     *  @return {Point}
     */

  }, {
    key: "toPublicKeyPoint",
    value: function toPublicKeyPoint() {
      return secp256k1.G.multiply(this.d);
    }
    /**
     *  @method toPublicKey
     *
     *  @return {PublicKey}
     */

  }, {
    key: "toPublicKey",
    value: function toPublicKey() {
      if (this.public_key) {
        return this.public_key;
      }

      this.public_key = _publicKey["default"].fromPoint(this.toPublicKeyPoint());
      return this.public_key;
    }
    /**
     *  @method toBuffer
     *
     *  @return {Buffer}
     */

  }, {
    key: "toBuffer",
    value: function toBuffer() {
      return this.d.toBuffer(32);
    }
    /** ECIES */

    /**
     *  @method getSharedSecret
     *  @param  {PublicKey}  publickey
     *  @param  {Boolean} legacy [default: false]
     *
     *  @return {Buffer}
     */

  }, {
    key: "getSharedSecret",
    value: function getSharedSecret(publickey) {
      var legacy = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      publickey = toPublic(publickey);
      var KB = publickey.toUncompressed().toBuffer();

      var KBP = _ecurve.Point.fromAffine(secp256k1, _bigi["default"].fromBuffer((0, _slice["default"])(KB).call(KB, 1, 33)), // x
      _bigi["default"].fromBuffer((0, _slice["default"])(KB).call(KB, 33, 65)) // y
      );

      var r = this.toBuffer();
      var P = KBP.multiply(_bigi["default"].fromBuffer(r));
      var S = P.affineX.toBuffer({
        size: 32
      });
      /*
       the input to sha512 must be exactly 32-bytes, to match the c++ implementation
       of getSharedSecret.  Right now S will be shorter if the most significant
       byte(s) is zero.  Pad it back to the full 32-bytes
       */

      if (!legacy && S.length < 32) {
        var _context;

        var pad = (0, _fill["default"])(_context = Buffer.alloc(32 - S.length)).call(_context, 0);
        S = (0, _concat["default"])(Buffer).call(Buffer, [pad, S]);
      } // SHA512 used in ECIES


      return (0, _hash.sha512)(S);
    }
    /**
     *  @method child
     *
     *  @param  {Number} offset
     *
     *  @throws {Error} - overflow of the key could not be derived
     *
     *  @return {PrivateKey}
     */

  }, {
    key: "child",
    value: function child(offset) {
      offset = (0, _concat["default"])(Buffer).call(Buffer, [this.toPublicKey().toBuffer(), offset]);
      offset = (0, _hash.sha256)(offset);

      var c = _bigi["default"].fromBuffer(offset);

      if (c.compareTo(n) >= 0) {
        throw new Error('Child offset went out of bounds, try again');
      }

      var derived = this.d.add(c); //	.mod(n)

      if (derived.signum() === 0) {
        throw new Error('Child offset derived to an invalid key, try again');
      }

      return new PrivateKey(derived);
    }
    /**
     *  @method fromHex
     *
     *  @param  {String} hex
     *
     *  @return {PrivateKey}
     */

  }, {
    key: "toHex",
    value:
    /**
     *  @method toHex
     *
     *  @return {String}
     */
    function toHex() {
      return this.toBuffer().toString('hex');
    }
  }], [{
    key: "fromBuffer",
    value: function fromBuffer(buf) {
      if (!Buffer.isBuffer(buf)) {
        throw new Error('Expecting paramter to be a Buffer type');
      }

      if (buf.length !== 32) {
        throw new Error("Expecting 32 bytes, instead got ".concat(buf.length));
      }

      if (buf.length === 0) {
        throw new Error('Empty buffer');
      }

      return new PrivateKey(_bigi["default"].fromBuffer(buf));
    }
    /**
     *  @method fromSeed
     *  This is private, the same seed produces the same private key every time.
     *
     *  @param {String} seed - any length string.
     *
     *   @return {PrivateKey}
     */

  }, {
    key: "fromSeed",
    value: function fromSeed(seed) {
      if (!(typeof seed === 'string')) {
        throw new Error('seed must be of type string');
      }

      return PrivateKey.fromBuffer((0, _hash.sha256)(seed));
    }
    /**
     *  @method fromWif
     *
     *  @param  {String} _privateWIF
     *
     *  @return {PrivateKey}
     */

  }, {
    key: "fromWif",
    value: function fromWif(_privateWIF) {
      var privateWIF = Buffer.from((0, _bs.decode)(_privateWIF));
      var version = privateWIF.readUInt8(0);

      _assert["default"].equal(0x80, version, "Expected version ".concat(0x80, ", instead got ", version)); // checksum includes the version


      var privateKey = (0, _slice["default"])(privateWIF).call(privateWIF, 0, -4);
      var checksum = (0, _slice["default"])(privateWIF).call(privateWIF, -4);
      var newChecksum = (0, _hash.sha256)(privateKey);
      newChecksum = (0, _hash.sha256)(newChecksum);
      newChecksum = (0, _slice["default"])(newChecksum).call(newChecksum, 0, 4);
      var isEqual = (0, _deepEqual["default"])(checksum, newChecksum); //	, 'Invalid checksum'

      if (!isEqual) {
        throw new Error('Checksum did not match');
      }

      privateKey = (0, _slice["default"])(privateKey).call(privateKey, 1);
      return PrivateKey.fromBuffer(privateKey);
    }
  }, {
    key: "fromHex",
    value: function fromHex(hex) {
      return PrivateKey.fromBuffer(Buffer.from(hex, 'hex'));
    }
  }]);
  return PrivateKey;
}();

var _default = PrivateKey;
exports["default"] = _default;