"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _bytebuffer = _interopRequireDefault(require("bytebuffer"));

var _bigi = _interopRequireDefault(require("bigi"));

var _ecurve = require("ecurve");

var _bs = require("bs58");

var _assert = _interopRequireDefault(require("assert"));

var _deepEqual = _interopRequireDefault(require("deep-equal"));

var _chainConfig = require("../../constants/chain-config");

var _hash = require("../hash");

var secp256k1 = (0, _ecurve.getCurveByName)('secp256k1');
var G = secp256k1.G,
    n = secp256k1.n;

var PublicKey = /*#__PURE__*/function () {
  /**
   *  @constructor
   *
   *  @param {Point} Q
   */
  function PublicKey(Q) {
    (0, _classCallCheck2["default"])(this, PublicKey);
    this.Q = Q;
  }
  /**
   *  @method fromBinary
   *
   *  @param  {String|Array} bin
   *
   *  @return {PublicKey}
   */


  (0, _createClass2["default"])(PublicKey, [{
    key: "toBuffer",
    value:
    /**
     *  @method toBuffer
     *
     *  @param  {Point|null} compressed
     *
     *  @return {Buffer}
     */
    function toBuffer() {
      var compressed = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.Q ? this.Q.compressed : null;

      if (this.Q === null) {
        return Buffer.from('000000000000000000000000000000000000000000000000000000000000000000', 'hex');
      }

      return this.Q.getEncoded(compressed);
    }
    /**
     *  @method fromPoint
     *
     *  @param  {Point} point
     *
     *  @return {PublicKey}
     */

  }, {
    key: "toUncompressed",
    value:
    /**
     *  @method toUncompressed
     *
     *  @return {PublicKey}
     */
    function toUncompressed() {
      var buf = this.Q.getEncoded(false);

      var point = _ecurve.Point.decodeFrom(secp256k1, buf);

      return PublicKey.fromPoint(point);
    }
    /**
     *  @method toBlockchainAddress
     *
     *  @return {String}
     */

  }, {
    key: "toBlockchainAddress",
    value: function toBlockchainAddress() {
      var pubBuf = this.toBuffer();
      var pubSha = (0, _hash.sha512)(pubBuf);
      return (0, _hash.ripemd160)(pubSha);
    }
    /**
     *  @method toString
     *  Alias for {@link toPublicKeyString}
     *
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {String}
     */

  }, {
    key: "toString",
    value: function toString() {
      var addressPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _chainConfig.ADDRESS_PREFIX;
      return this.toPublicKeyString(addressPrefix);
    }
    /**
     *  @method toPublicKeyString
     *  Full public key
     *
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {String}
     */

  }, {
    key: "toPublicKeyString",
    value: function toPublicKeyString() {
      var addressPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _chainConfig.ADDRESS_PREFIX;
      var pubBuf = this.toBuffer();
      var checksum = (0, _hash.ripemd160)(pubBuf);
      var addy = (0, _concat["default"])(Buffer).call(Buffer, [pubBuf, (0, _slice["default"])(checksum).call(checksum, 0, 4)]);
      return addressPrefix + (0, _bs.encode)(addy);
    }
    /**
     *  @method fromPublicKeyString
     *
     *  @param  {String} publicKey
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {PublicKey|null} (if the publicKey string is invalid)
     */

  }, {
    key: "toAddressString",
    value:
    /**
     *  @method toAddressString
     *
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {String}
     */
    function toAddressString() {
      var addressPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _chainConfig.ADDRESS_PREFIX;
      var pubBuf = this.toBuffer();
      var pubSha = (0, _hash.sha512)(pubBuf);
      var addy = (0, _hash.ripemd160)(pubSha);
      var checksum = (0, _hash.ripemd160)(addy);
      addy = (0, _concat["default"])(Buffer).call(Buffer, [addy, (0, _slice["default"])(checksum).call(checksum, 0, 4)]);
      return addressPrefix + (0, _bs.encode)(addy);
    }
    /**
     *  @method toPtsAddy
     */

  }, {
    key: "toPtsAddy",
    value: function toPtsAddy() {
      var pubBuf = this.toBuffer();
      var pubSha = (0, _hash.sha256)(pubBuf);
      var addy = (0, _hash.ripemd160)(pubSha);
      addy = (0, _concat["default"])(Buffer).call(Buffer, [Buffer.from([0x38]), addy]); //	version 56(decimal)

      var checksum = (0, _hash.sha256)(addy);
      checksum = (0, _hash.sha256)(checksum);
      addy = (0, _concat["default"])(Buffer).call(Buffer, [addy, (0, _slice["default"])(checksum).call(checksum, 0, 4)]);
      return (0, _bs.encode)(addy);
    }
    /**
     *	@method child
     *
     *  @param  {Buffer} offset
     *
     *  @return {PublicKey}
     */

  }, {
    key: "child",
    value: function child(offset) {
      (0, _assert["default"])(Buffer.isBuffer(offset), 'Buffer required: offset');

      _assert["default"].equal(offset.length, 32, 'offset length');

      offset = (0, _concat["default"])(Buffer).call(Buffer, [this.toBuffer(), offset]);
      offset = (0, _hash.sha256)(offset);

      var c = _bigi["default"].fromBuffer(offset);

      if (c.compareTo(n) >= 0) {
        throw new Error('Child offset went out of bounds, try again');
      }

      var cG = G.multiply(c);
      var Qprime = this.Q.add(cG);

      if (secp256k1.isInfinity(Qprime)) {
        throw new Error('Child offset derived to an invalid key, try again');
      }

      return PublicKey.fromPoint(Qprime);
    }
    /**
     *  @method toByteBuffer
     *
     *  @return {ByteBuffer}
     */

  }, {
    key: "toByteBuffer",
    value: function toByteBuffer() {
      var b = new _bytebuffer["default"](_bytebuffer["default"].DEFAULT_CAPACITY, _bytebuffer["default"].LITTLE_ENDIAN);
      this.appendByteBuffer(b);
      return b.copy(0, b.offset);
    }
    /**
     *  @method fromHex
     *
     *  @param  {String} hex
     *
     *  @return {PublicKey}
     */

  }, {
    key: "toHex",
    value:
    /**
     *  @method toHex
     *
     *  @return {String}
     */
    function toHex() {
      return this.toBuffer().toString('hex');
    }
    /**
     *  @method fromPublicKeyStringHex
     *
     *  @param  {String} hex
     *
     *  @return {PublicKey}
     */

  }], [{
    key: "fromBinary",
    value: function fromBinary(bin) {
      return PublicKey.fromBuffer(Buffer.from(bin, 'binary'));
    }
    /**
     *  @method fromBuffer
     *
     *  @param  {Buffer} buffer
     *
     *  @return {PublicKey}
     */

  }, {
    key: "fromBuffer",
    value: function fromBuffer(buffer) {
      if (buffer.toString('hex') === '000000000000000000000000000000000000000000000000000000000000000000') {
        return new PublicKey(null);
      }

      return new PublicKey(_ecurve.Point.decodeFrom(secp256k1, buffer));
    }
  }, {
    key: "fromPoint",
    value: function fromPoint(point) {
      return new PublicKey(point);
    }
  }, {
    key: "fromPublicKeyString",
    value: function fromPublicKeyString(publicKey) {
      var addressPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _chainConfig.ADDRESS_PREFIX;

      try {
        return PublicKey.fromStringOrThrow(publicKey, addressPrefix);
      } catch (e) {
        return null;
      }
    }
    /**
     *  @method fromStringOrThrow
     *
     *  @param  {String} publicKey
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @throws {Error} if public key is invalid
     *
     *  @return {PublicKey}
     */

  }, {
    key: "fromStringOrThrow",
    value: function fromStringOrThrow(publicKey) {
      var _context;

      var addressPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _chainConfig.ADDRESS_PREFIX;
      var prefix = (0, _slice["default"])(publicKey).call(publicKey, 0, addressPrefix.length);

      _assert["default"].equal(addressPrefix, prefix, (0, _concat["default"])(_context = "Expecting key to begin with ".concat(addressPrefix, ", instead got ")).call(_context, prefix));

      publicKey = (0, _slice["default"])(publicKey).call(publicKey, addressPrefix.length);
      publicKey = Buffer.from((0, _bs.decode)(publicKey), 'binary');
      var checksum = (0, _slice["default"])(publicKey).call(publicKey, -4);
      publicKey = (0, _slice["default"])(publicKey).call(publicKey, 0, -4);
      var newChecksum = (0, _hash.ripemd160)(publicKey);
      newChecksum = (0, _slice["default"])(newChecksum).call(newChecksum, 0, 4);
      var isEqual = (0, _deepEqual["default"])(checksum, newChecksum); //	, 'Invalid checksum'

      if (!isEqual) {
        throw new Error('Checksum did not match');
      }

      return PublicKey.fromBuffer(publicKey);
    }
  }, {
    key: "fromHex",
    value: function fromHex(hex) {
      return PublicKey.fromBuffer(Buffer.from(hex, 'hex'));
    }
  }, {
    key: "fromPublicKeyStringHex",
    value: function fromPublicKeyStringHex(hex) {
      return PublicKey.fromPublicKeyString(Buffer.from(hex, 'hex'));
    }
  }]);
  return PublicKey;
}();

var _default = PublicKey;
exports["default"] = _default;