"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _aes = _interopRequireDefault(require("crypto-js/aes"));

var _encHex = _interopRequireDefault(require("crypto-js/enc-hex"));

var _encBase = _interopRequireDefault(require("crypto-js/enc-base64"));

var _assert = _interopRequireDefault(require("assert"));

var _hash2 = require("./hash");

/** Provides symetric encrypt and decrypt via AES. */
var Aes = /*#__PURE__*/function () {
  /**
   *  @constructor
   *
   *  @param {Buffer} iv
   *  @param {Buffer} key
   */
  function Aes(iv, key) {
    (0, _classCallCheck2["default"])(this, Aes);
    this.iv = iv;
    this.key = key;
  }
  /**
   *  @method clear
   *  This is an excellent way to ensure that all references to Aes can not operate anymore
   *  (example: a wallet becomes locked). An application should ensure
   *  there is only one Aes object instance for a given secret `seed`.
   */


  (0, _createClass2["default"])(Aes, [{
    key: "clear",
    value: function clear() {
      this.iv = undefined;
      this.key = undefined;
      return undefined;
    }
    /**
     *  @method fromSeed
     *
     *  @param  {String} seed [secret seed may be used to encrypt or decrypt]
     *
     *  @return {Aes}
     */

  }, {
    key: "_decryptWordArray",
    value:
    /** @private */
    function _decryptWordArray(cipher) {
      // https://code.google.com/p/crypto-js/#Custom_Key_and_IV
      // see wallet_records.cpp master_key::decrypt_key
      return _aes["default"].decrypt({
        ciphertext: cipher,
        salt: null
      }, this.key, {
        iv: this.iv
      });
    }
    /** @private */

  }, {
    key: "_encryptWordArray",
    value: function _encryptWordArray(plaintext) {
      //	https://code.google.com/p/crypto-js/issues/detail?id=85
      var cipher = _aes["default"].encrypt(plaintext, this.key, {
        iv: this.iv
      });

      return _encBase["default"].parse(cipher.toString());
    }
    /**
     *  @method decrypt
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} ciphertext
     *
     *  @return {Buffer}
     */

  }, {
    key: "decrypt",
    value: function decrypt(ciphertext) {
      if (typeof ciphertext === 'string') {
        ciphertext = Buffer.from(ciphertext, 'binary');
      }

      if (!Buffer.isBuffer(ciphertext)) {
        throw new Error('buffer required');
      }

      (0, _assert["default"])(ciphertext, 'Missing cipher text'); // hex is the only common format

      var hex = this.decryptHex(ciphertext.toString('hex'));
      return Buffer.from(hex, 'hex');
    }
    /**
     *  @method encrypt
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} plaintext
     *
     *  @return {Buffer}
     */

  }, {
    key: "encrypt",
    value: function encrypt(plaintext) {
      if (typeof plaintext === 'string') {
        plaintext = Buffer.from(plaintext, 'binary');
      }

      if (!Buffer.isBuffer(plaintext)) {
        throw new Error('buffer required');
      } // assert plaintext, 'Missing plain text'
      // hex is the only common format


      var hex = this.encryptHex(plaintext.toString('hex'));
      return Buffer.from(hex, 'hex');
    }
    /**
     *  @method encryptToHex
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String|Buffer} plaintext
     *
     *  @return {String}
     */

  }, {
    key: "encryptToHex",
    value: function encryptToHex(plaintext) {
      if (typeof plaintext === 'string') {
        plaintext = Buffer.from(plaintext, 'binary');
      }

      if (!Buffer.isBuffer(plaintext)) {
        throw new Error('buffer required');
      } //	assert plaintext, 'Missing plain text'
      // hex is the only common format


      return this.encryptHex(plaintext.toString('hex'));
    }
    /**
     *  @method decryptHex
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} cipher - hex
     *
     *  @return {String} (could easily be readable text)
     */

  }, {
    key: "decryptHex",
    value: function decryptHex(cipher) {
      (0, _assert["default"])(cipher, 'Missing cipher text'); // Convert data into word arrays (used by Crypto)

      var cipherArray = _encHex["default"].parse(cipher);

      var plainwords = this._decryptWordArray(cipherArray);

      return _encHex["default"].stringify(plainwords);
    }
    /**
     *  @method decryptHexToBuffer
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} cipher - hex
     *
     *  @return {Buffer} - encoded as specified by the parameter
     */

  }, {
    key: "decryptHexToBuffer",
    value: function decryptHexToBuffer(cipher) {
      (0, _assert["default"])(cipher, 'Missing cipher text'); // Convert data into word arrays (used by Crypto)

      var cipherArray = _encHex["default"].parse(cipher);

      var plainwords = this._decryptWordArray(cipherArray);

      var plainhex = _encHex["default"].stringify(plainwords);

      return Buffer.from(plainhex, 'hex');
    }
    /**
     *  @method decryptHexToText
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} cipher - hex
     *  @param  {String} [encoding = 'binary'] - a valid Buffer encoding
     *
     *  @return {String} - encoded as specified by the parameter
     */

  }, {
    key: "decryptHexToText",
    value: function decryptHexToText(cipher) {
      var encoding = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'binary';
      return this.decryptHexToBuffer(cipher).toString(encoding);
    }
    /**
     *  @method encryptHex
     *  This method does not use a checksum, the returned data must be validated some other way.
     *
     *  @param  {String} plainhex - hex
     *
     *  @return {String} - hex
     */

  }, {
    key: "encryptHex",
    value: function encryptHex(plainhex) {
      var plainArray = _encHex["default"].parse(plainhex);

      var cipherArray = this._encryptWordArray(plainArray);

      return _encHex["default"].stringify(cipherArray);
    }
  }], [{
    key: "fromSeed",
    value: function fromSeed(seed) {
      if (seed === undefined) {
        throw new Error('seed is required');
      }

      var _hash = (0, _hash2.sha512)(seed);

      _hash = _hash.toString('hex');
      return Aes.fromSha512(_hash);
    }
    /**
     *  @method fromSha512
     *
     *  @param  {String} hash A 128 byte hex string,
     *  	typically one would call {@link fromSeed} instead
     *
     *  @return {Aes}
     */

  }, {
    key: "fromSha512",
    value: function fromSha512(hash) {
      _assert["default"].equal(hash.length, 128, "A Sha512 in HEX should be 128 characters long, instead got ".concat(hash.length));

      var iv = _encHex["default"].parse(hash.substring(64, 96));

      var key = _encHex["default"].parse(hash.substring(0, 64));

      return new Aes(iv, key);
    }
    /**
     *  @method fromBuffer
     *
     *  @param  {Buffer} buf
     *
     *  @return {Aes}
     */

  }, {
    key: "fromBuffer",
    value: function fromBuffer(buf) {
      (0, _assert["default"])(Buffer.isBuffer(buf), 'Expecting Buffer');

      _assert["default"].equal(buf.length, 64, "A Sha512 Buffer should be 64 characters long, instead got ".concat(buf.length));

      return Aes.fromSha512(buf.toString('hex'));
    }
    /**
     *  @method decryptWithChecksum
     *
     *  @throws {Error} - 'Invalid Key, ...'
     *
     *  @param {PrivateKey} privateKey - required and used for decryption
     *  @param {PublicKey} publicKey - required and used to calcualte the shared secret
     *  @param {String} [nonce = ''] optional but should always be provided and
     *  	be unique when re-using the same private/public keys more than once.
     *  	This nonce is not a secret.
     *  @param {String|Buffer} message - Encrypted message containing a checksum
     *
     *  @return {Buffer}
     */

  }, {
    key: "decryptWithChecksum",
    value: function decryptWithChecksum(privateKey, publicKey, nonce, message) {
      var legacy = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      // Warning: Do not put `nonce = ''` in the arguments,
      // in es6 this will not convert 'null' into an emtpy string
      // nonce null or undefined
      if (nonce === null || nonce === undefined) {
        nonce = '';
      }

      if (!Buffer.isBuffer(message)) {
        message = Buffer.from(message, 'hex');
      }

      var S = privateKey.getSharedSecret(publicKey, legacy);
      var aes = Aes.fromSeed((0, _concat["default"])(Buffer).call(Buffer, [// A null or empty string nonce will not effect the hash
      Buffer.from(String(nonce)), Buffer.from(S.toString('hex'))]));
      var planebuffer = aes.decrypt(message);

      if (!(planebuffer.length >= 4)) {
        throw new Error('Invalid key, could not decrypt message(1)');
      }

      var checksum = (0, _slice["default"])(planebuffer).call(planebuffer, 0, 4);
      var plaintext = (0, _slice["default"])(planebuffer).call(planebuffer, 4);
      var newChecksum = (0, _hash2.sha256)(plaintext);
      newChecksum = (0, _slice["default"])(newChecksum).call(newChecksum, 0, 4);
      newChecksum = newChecksum.toString('hex');

      if (!(checksum.toString('hex') === newChecksum)) {
        throw new Error('Invalid key, could not decrypt message(2)');
      }

      return plaintext;
    }
    /**
     *  @method encryptWithChecksum
     *  Identical to {@link decryptWithChecksum} but used to encrypt.  Should not throw an error.
     *
     *  @param  {PrivateKey} privateKey
     *  @param  {PublicKey} publicKey
     *  @param  {String} nonce
     *  @param  {String|Buffer} message
     *
     *  @return {Buffer} - Encrypted message which includes a checksum
     */

  }, {
    key: "encryptWithChecksum",
    value: function encryptWithChecksum(privateKey, publicKey, nonce, message) {
      var _context;

      // Warning: Do not put `nonce = ''` in the arguments,
      // in es6 this will not convert 'null' into an emtpy string
      // nonce null or undefined
      if (nonce === null || nonce === undefined) {
        nonce = '';
      }

      if (!Buffer.isBuffer(message)) {
        message = Buffer.from(message, 'binary');
      }

      var S = privateKey.getSharedSecret(publicKey);
      var aes = Aes.fromSeed((0, _concat["default"])(Buffer).call(Buffer, [// A null or empty string nonce will not effect the hash
      Buffer.from(String(nonce)), Buffer.from(S.toString('hex'))]));
      var checksum = (0, _slice["default"])(_context = (0, _hash2.sha256)(message)).call(_context, 0, 4);
      var payload = (0, _concat["default"])(Buffer).call(Buffer, [checksum, message]);
      return aes.encrypt(payload);
    }
  }]);
  return Aes;
}();

var _default = Aes;
exports["default"] = _default;