"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.calcPubKeyRecoveryParam = calcPubKeyRecoveryParam;
exports.deterministicGenerateK = deterministicGenerateK;
exports.sign = sign;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _fill = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/fill"));

var _bigi = _interopRequireDefault(require("bigi"));

var _hash = require("./hash");

var _ECSignature = _interopRequireDefault(require("./ECSignature"));

var _validators = require("../utils/validators");

// FIXME: jsdoc

/**
 * @param {*} curve
 * @param {Buffer} hash
 * @param {BigInteger} d
 * @param {*} checkSig
 * @param {number} nonce
 */
function deterministicGenerateK(curve, hash, d, checkSig, nonce) {
  if (nonce) hash = (0, _hash.sha256)((0, _concat["default"])(Buffer).call(Buffer, [hash, Buffer.alloc(nonce)])); // sanity transaction.js

  if (hash.length !== 32) throw new Error('invalid sha256 hash length');
  var x = d.toBuffer(32);
  var k = Buffer.alloc(32);
  var v = Buffer.alloc(32); // Step B

  (0, _fill["default"])(v).call(v, 1); // Step C

  (0, _fill["default"])(k).call(k, 0); // Step D

  k = (0, _hash.HmacSHA256)((0, _concat["default"])(Buffer).call(Buffer, [v, Buffer.from([0]), x, hash]), k); // Step E

  v = (0, _hash.HmacSHA256)(v, k); // Step F

  k = (0, _hash.HmacSHA256)((0, _concat["default"])(Buffer).call(Buffer, [v, Buffer.from([1]), x, hash]), k); // Step G

  v = (0, _hash.HmacSHA256)(v, k); // Step H1/H2a, ignored as tlen === qlen (256 bit)
  // Step H2b

  v = (0, _hash.HmacSHA256)(v, k); // Step H3, repeat until T is within the interval [1, n - 1]

  for (var T = _bigi["default"].fromBuffer(v);;) {
    if (T.signum() > 0 && T.compareTo(curve.n) < 0 && checkSig(T)) return T;
    k = (0, _hash.HmacSHA256)((0, _concat["default"])(Buffer).call(Buffer, [v, Buffer.from([0])]), k);
    v = (0, _hash.HmacSHA256)(v, k); // Step H1/H2a, again, ignored as tlen === qlen (256 bit)
    // Step H2b again

    v = (0, _hash.HmacSHA256)(v, k);
    T = _bigi["default"].fromBuffer(v);
  }
} // FIXME: jsdoc

/**
 * @param {*} curve
 * @param {Buffer} hash
 * @param {*} d
 * @param {number} nonce
 */


function sign(curve, hash, d, nonce) {
  var e = _bigi["default"].fromBuffer(hash);

  var n = curve.n,
      G = curve.G;
  var r;
  var s;
  deterministicGenerateK(curve, hash, d, function (k) {
    // find canonically valid signature
    var Q = G.multiply(k);
    if (curve.isInfinity(Q)) return false;
    r = Q.affineX.mod(n);
    if (r.signum() === 0) return false;
    s = k.modInverse(n).multiply(e.add(d.multiply(r))).mod(n);
    if (s.signum() === 0) return false;
    return true;
  }, nonce);
  var N_OVER_TWO = n.shiftRight(1); // enforce low S values, see bip62: 'low s values in signatures'

  if (s.compareTo(N_OVER_TWO) > 0) s = n.subtract(s);
  return new _ECSignature["default"](r, s);
} // FIXME: jsdoc

/**
 * @param {*} curve
 * @param {*} e
 * @param {ECSignature} signature
 * @param {number} i
 */


function recoverPubKey(curve, e, signature, i) {
  (0, _validators.validateUnsignedSafeInteger)(i);
  if (i > 3) throw new Error('Recovery param is more than two bits');
  var n = curve.n,
      G = curve.G;
  var r = signature.r,
      s = signature.s;
  if (r.signum() <= 0 || r.compareTo(n) >= 0) throw new Error('invalid r-component');
  if (s.signum() <= 0 || s.compareTo(n) >= 0) throw new Error('invalid s-component'); // A set LSB signifies that the y-coordinate is odd

  var isYOdd = i % 2; // The more significant bit specifies whether we should use the
  // first or second candidate key.

  var isSecondKey = Math.floor(i / 2); // 1.1 Let x = r + jn

  var x = isSecondKey ? r.add(n) : r;
  var R = curve.pointFromX(isYOdd, x); // 1.4 Check that nR is at infinity

  var nR = R.multiply(n);
  if (!curve.isInfinity(nR)) throw new Error('nR is not a valid curve point'); // Compute -e = require(e

  var eNeg = e.negate().mod(n); // 1.6.1 Compute Q = r^-1 * (sR-eG)

  var rInv = r.modInverse(n);
  var Q = R.multiplyTwo(s, G, eNeg).multiply(rInv);
  curve.validate(Q);
  return Q;
}

function calcPubKeyRecoveryParam(curve, e, signature, Q) {
  for (var i = 0; i < 4; i += 1) {
    var Qprime = recoverPubKey(curve, e, signature, i); // 1.6.2 Verify Q

    if (Qprime.equals(Q)) return i;
  }

  throw new Error('Unable to find valid recovery factor');
}