"use strict";

var _typeof = require("@babel/runtime-corejs3/helpers/typeof");

var _WeakMap = require("@babel/runtime-corejs3/core-js-stable/weak-map");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _Object$getOwnPropertyDescriptor = require("@babel/runtime-corejs3/core-js-stable/object/get-own-property-descriptor");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _bigi = _interopRequireDefault(require("bigi"));

var _bs = require("bs58");

var _deepEqual = _interopRequireDefault(require("deep-equal"));

var _assert = _interopRequireDefault(require("assert"));

var ed25519 = _interopRequireWildcard(require("ed25519.js"));

var _hash = require("./hash");

var _publicKey = _interopRequireDefault(require("./public-key"));

function _getRequireWildcardCache(nodeInterop) { if (typeof _WeakMap !== "function") return null; var cacheBabelInterop = new _WeakMap(); var cacheNodeInterop = new _WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = _Object$defineProperty && _Object$getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? _Object$getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { _Object$defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var PrivateKey = /*#__PURE__*/function () {
  /**
   *  @constructor
   *
   *  @param {BigInteger} d
   */
  function PrivateKey(d) {
    (0, _classCallCheck2["default"])(this, PrivateKey);
    this.d = d;
  }
  /**
   *  @method fromBuffer
   *
   *  @param  {Buffer} buf
   *
   *  @return {PrivateKey}
   */


  (0, _createClass2["default"])(PrivateKey, [{
    key: "toWif",
    value:
    /**
     *  @method toWif
     *
     *  @return {String}
     */
    function toWif() {
      var privateKey = this.toBuffer(); // checksum includes the version

      privateKey = (0, _concat["default"])(Buffer).call(Buffer, [Buffer.from([0x80]), privateKey]);
      var checksum = (0, _hash.sha256)(privateKey);
      checksum = (0, _hash.sha256)(checksum);
      checksum = (0, _slice["default"])(checksum).call(checksum, 0, 4);
      var privateWIF = (0, _concat["default"])(Buffer).call(Buffer, [privateKey, checksum]);
      return (0, _bs.encode)(privateWIF);
    }
    /**
     *  @method toPublicKey
     *
     *  @return {PublicKey}
     */

  }, {
    key: "toPublicKey",
    value: function toPublicKey() {
      if (this.public_key) {
        return this.public_key;
      }

      var publicKey = ed25519.derivePublicKey(this.toBuffer());
      this.public_key = _publicKey["default"].fromBuffer(publicKey);
      return this.public_key;
    }
    /**
     *  @method toBuffer
     *
     *  @return {Buffer}
     */

  }, {
    key: "toBuffer",
    value: function toBuffer() {
      return this.d.toBuffer(32);
    }
    /** ECIES */

    /**
     *  @method getSharedSecret
     *
     *  @return {Buffer}
     */

  }, {
    key: "getSharedSecret",
    value: function getSharedSecret() {
      throw new Error('Need to implement');
    }
    /**
     *  @method fromHex
     *
     *  @param  {String} hex
     *
     *  @return {PrivateKey}
     */

  }, {
    key: "toHex",
    value:
    /**
     *  @method toHex
     *
     *  @return {String}
     */
    function toHex() {
      return this.toBuffer().toString('hex');
    }
    /**
     *  @method toPrivateKeyString
     *  Full public key
     *
     *
     *  @return {String}
     */

  }, {
    key: "toPrivateKeyString",
    value: function toPrivateKeyString() {
      var buf = this.toBuffer();
      return (0, _bs.encode)(buf);
    }
    /**
     *  @method fromPublicKeyString
     *
     *  @param  {String} privateKeyString
     *
     *  @return {PrivateKey|null} (if the private string is invalid)
     */

  }], [{
    key: "fromBuffer",
    value: function fromBuffer(buf) {
      if (!Buffer.isBuffer(buf)) {
        throw new Error('Expecting paramter to be a Buffer type');
      }

      if (buf.length !== 32) {
        throw new Error("Expecting 32 bytes, instead got ".concat(buf.length));
      }

      if (buf.length === 0) {
        throw new Error('Empty buffer');
      }

      return new PrivateKey(_bigi["default"].fromBuffer(buf));
    }
    /**
     *  @method fromSeed
     *  This is private, the same seed produces the same private key every time.
     *
     *  @param {String} seed - any length string.
     *
     *   @return {PrivateKey}
     */

  }, {
    key: "fromSeed",
    value: function fromSeed(seed) {
      if (!(typeof seed === 'string')) {
        throw new Error('seed must be of type string');
      }

      return PrivateKey.fromBuffer((0, _hash.sha256)(seed));
    }
    /**
     *  @method fromWif
     *
     *  @param  {String} _privateWIF
     *
     *  @return {PrivateKey}
     */

  }, {
    key: "fromWif",
    value: function fromWif(_privateWIF) {
      var privateWIF = Buffer.from((0, _bs.decode)(_privateWIF));
      var version = privateWIF.readUInt8(0);

      _assert["default"].equal(0x80, version, "Expected version ".concat(0x80, ", instead got ", version)); // checksum includes the version


      var privateKey = (0, _slice["default"])(privateWIF).call(privateWIF, 0, -4);
      var checksum = (0, _slice["default"])(privateWIF).call(privateWIF, -4);
      var newChecksum = (0, _hash.sha256)(privateKey);
      newChecksum = (0, _hash.sha256)(newChecksum);
      newChecksum = (0, _slice["default"])(newChecksum).call(newChecksum, 0, 4);
      var isEqual = (0, _deepEqual["default"])(checksum, newChecksum); //	, 'Invalid checksum'

      if (!isEqual) {
        throw new Error('Checksum did not match');
      }

      privateKey = (0, _slice["default"])(privateKey).call(privateKey, 1);
      return PrivateKey.fromBuffer(privateKey);
    }
  }, {
    key: "fromHex",
    value: function fromHex(hex) {
      return PrivateKey.fromBuffer(Buffer.from(hex, 'hex'));
    }
  }, {
    key: "fromPrivateKeyString",
    value: function fromPrivateKeyString(privateKeyString) {
      var privateKeyBuffer = Buffer.from((0, _bs.decode)(privateKeyString), 'binary');
      return PrivateKey.fromBuffer(privateKeyBuffer);
    }
  }]);
  return PrivateKey;
}();

var _default = PrivateKey;
exports["default"] = _default;