"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _bytebuffer = _interopRequireDefault(require("bytebuffer"));

var _bs = require("bs58");

var _assert = _interopRequireDefault(require("assert"));

var _chainConfig = require("../constants/chain-config");

var _hash = require("./hash");

var PublicKey = /*#__PURE__*/function () {
  /**
   *  @constructor
   *
   *  @param {Buffer} buffer
   */
  function PublicKey(buffer) {
    (0, _classCallCheck2["default"])(this, PublicKey);
    this.buffer = buffer;
  }
  /**
   *  @method fromBinary
   *
   *  @param  {String|Array} bin
   *
   *  @return {PublicKey}
   */


  (0, _createClass2["default"])(PublicKey, [{
    key: "toBuffer",
    value:
    /**
     *  @method toBuffer
     *
     *  @return {Buffer}
     */
    function toBuffer() {
      if (this.buffer === null) {
        return Buffer.from('000000000000000000000000000000000000000000000000000000000000000000', 'hex');
      }

      return this.buffer;
    }
    /**
     *  @method toBlockchainAddress
     *
     *  @return {String}
     */

  }, {
    key: "toBlockchainAddress",
    value: function toBlockchainAddress() {
      var pubBuf = this.toBuffer();
      var pubSha = (0, _hash.sha512)(pubBuf);
      return (0, _hash.ripemd160)(pubSha);
    }
    /**
     *  @method toString
     *  Alias for {@link toPublicKeyString}
     *
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {String}
     */

  }, {
    key: "toString",
    value: function toString() {
      var addressPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _chainConfig.ADDRESS_PREFIX;
      return this.toPublicKeyString(addressPrefix);
    }
    /**
     *  @method toPublicKeyString
     *  Full public key
     *
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {String}
     */

  }, {
    key: "toPublicKeyString",
    value: function toPublicKeyString() {
      var addressPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _chainConfig.ADDRESS_PREFIX;
      var pubBuf = this.toBuffer();
      return addressPrefix + (0, _bs.encode)(pubBuf);
    }
    /**
     *  @method fromPublicKeyString
     *
     *  @param  {String} publicKey
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @return {PublicKey|null} (if the publicKey string is invalid)
     */

  }, {
    key: "toByteBuffer",
    value:
    /**
     *  @method toByteBuffer
     *
     *  @return {ByteBuffer}
     */
    function toByteBuffer() {
      var b = new _bytebuffer["default"](_bytebuffer["default"].DEFAULT_CAPACITY, _bytebuffer["default"].LITTLE_ENDIAN);
      this.appendByteBuffer(b);
      return b.copy(0, b.offset);
    }
    /**
     *  @method fromHex
     *
     *  @param  {String} hex
     *
     *  @return {PublicKey}
     */

  }, {
    key: "toHex",
    value:
    /**
     *  @method toHex
     *
     *  @return {String}
     */
    function toHex() {
      return this.toBuffer().toString('hex');
    }
    /**
     *  @method fromPublicKeyStringHex
     *
     *  @param  {String} hex
     *
     *  @return {PublicKey}
     */

  }], [{
    key: "fromBinary",
    value: function fromBinary(bin) {
      return PublicKey.fromBuffer(Buffer.from(bin, 'binary'));
    }
    /**
     *  @method fromBuffer
     *
     *  @param  {Buffer} buffer
     *
     *  @return {PublicKey}
     */

  }, {
    key: "fromBuffer",
    value: function fromBuffer(buffer) {
      if (buffer.toString('hex') === '000000000000000000000000000000000000000000000000000000000000000000') {
        return new PublicKey(null);
      }

      return new PublicKey(buffer);
    }
  }, {
    key: "fromPublicKeyString",
    value: function fromPublicKeyString(publicKey) {
      var addressPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _chainConfig.ADDRESS_PREFIX;

      try {
        return PublicKey.fromStringOrThrow(publicKey, addressPrefix);
      } catch (e) {
        console.error('error', e);
        return null;
      }
    }
    /**
     *  @method fromStringOrThrow
     *
     *  @param  {String} publicKey
     *  @param  {String} [addressPrefix=CHAIN_CONFIG.ADDRESS_PREFIX]
     *
     *  @throws {Error} if public key is invalid
     *
     *  @return {PublicKey}
     */

  }, {
    key: "fromStringOrThrow",
    value: function fromStringOrThrow(publicKey) {
      var _context;

      var addressPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _chainConfig.ADDRESS_PREFIX;
      var prefix = (0, _slice["default"])(publicKey).call(publicKey, 0, addressPrefix.length);

      _assert["default"].equal(addressPrefix, prefix, (0, _concat["default"])(_context = "Expecting key to begin with ".concat(addressPrefix, ", instead got ")).call(_context, prefix));

      publicKey = (0, _slice["default"])(publicKey).call(publicKey, addressPrefix.length);
      publicKey = Buffer.from((0, _bs.decode)(publicKey), 'binary');
      return PublicKey.fromBuffer(publicKey);
    }
  }, {
    key: "fromHex",
    value: function fromHex(hex) {
      return PublicKey.fromBuffer(Buffer.from(hex, 'hex'));
    }
  }, {
    key: "fromPublicKeyStringHex",
    value: function fromPublicKeyStringHex(hex) {
      return PublicKey.fromPublicKeyString(Buffer.from(hex, 'hex'));
    }
  }]);
  return PublicKey;
}();

var _default = PublicKey;
exports["default"] = _default;