"use strict";

var _Reflect$construct = require("@babel/runtime-corejs3/core-js-stable/reflect/construct");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/getPrototypeOf"));

var _constants = require("../../constants");

var _wsConstants = require("../../constants/ws-constants");

var _baseApi = _interopRequireDefault(require("./base-api"));

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = _Reflect$construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !_Reflect$construct) return false; if (_Reflect$construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(_Reflect$construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

/** @typedef {import("../providers").WsProvider} WsProvider */

/** @typedef {import("../providers").HttpProvider} HttpProvider */

/** @typedef {"" | "eth" | "btc"} SidechainType */

/** @typedef {import("../../../types/interfaces/chain").OperationHistoryObject} OperationHistoryObject */

/** @typedef {import("../../serializers/basic/integers")} Integer_t */

/** @typedef {import("../../serializers/chain/id/protocol")["contractId"]["__TOutput__"]} ContractId */
var HistoryAPI = /*#__PURE__*/function (_BaseEchoApi) {
  (0, _inherits2["default"])(HistoryAPI, _BaseEchoApi);

  var _super = _createSuper(HistoryAPI);

  /**
   * @constructor
   * @param {WsProvider | HttpProvider} provider
   */
  function HistoryAPI(provider) {
    (0, _classCallCheck2["default"])(this, HistoryAPI);
    return _super.call(this, provider, _wsConstants.CHAIN_API.HISTORY_API);
  }
  /**
   *  @method getAccountHistory
   *  Get operations relevant to the specificed account.
   *
   *  @param {String} accountId
   *  @param {String} stop [Id of the earliest operation to retrieve]
   *  @param {Number} limit     [count operations (max 100)]
   *  @param {String} start [Id of the most recent operation to retrieve]
   *
   *  @return {Promise}
   */


  (0, _createClass2["default"])(HistoryAPI, [{
    key: "getAccountHistory",
    value: function getAccountHistory(accountId) {
      var stop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _constants.START_OPERATION_ID;
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
      var start = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _constants.START_OPERATION_ID;
      return this.exec('get_account_history', [accountId, stop, limit, start]);
    }
    /**
     *  @method getRelativeAccountHistory
     *  Get operations relevant to the specified account referenced
     *  by an event numbering specific to the account.
     *
     *  @param {String} accountId
     *  @param {Number} stop [Sequence number of earliest operation]
     *  @param {Number} limit     [count operations (max 100)]
     *  @param {Number} start [Sequence number of the most recent operation to retrieve]
     *
     *  @return {Promise}
     */

  }, {
    key: "getRelativeAccountHistory",
    value: function getRelativeAccountHistory(accountId) {
      var stop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
      var start = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
      return this.exec('get_relative_account_history', [accountId, stop, limit, start]);
    }
    /**
     *  @method getAccountHistoryOperations
     *  Get only asked operations relevant to the specified account.
     *
     *  @param {String} accountId
     *  @param {String} operationId
     *  @param {Number} start [Id of the most recent operation to retrieve]
     *  @param {Number} stop [Id of the earliest operation to retrieve]
     *  @param {Number} limit     [count operations (max 100)]
     *
     *  @return {Promise}
     */

  }, {
    key: "getAccountHistoryOperations",
    value: function getAccountHistoryOperations(accountId, operationId) {
      var start = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _constants.START_OPERATION_ID;
      var stop = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _constants.START_OPERATION_ID;
      var limit = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 100;
      return this.exec('get_account_history_operations', [accountId, operationId, start, stop, limit]);
    }
    /**
     *  @method getContractHistory
     *  Get operations relevant to the specificed account.
     *
     *  @param {String} contractId
     *  @param {String} stop [Id of the earliest operation to retrieve]
     *  @param {Number} limit     [count operations (max 100)]
     *  @param {String} start [Id of the most recent operation to retrieve]
     *
     *  @return {Promise}
     */

  }, {
    key: "getContractHistory",
    value: function getContractHistory(contractId) {
      var stop = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _constants.START_OPERATION_ID;
      var limit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
      var start = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _constants.START_OPERATION_ID;
      return this.exec('get_contract_history', [contractId, stop, limit, start]);
    }
    /**
     * @param {ContractId} contract
     * @param {Integer_t["uint64"]["__TOutput__"]} stop
     * @param {Integer_t["uint32"]["__TOutput__"]} limit
     * @param {Integer_t["uint64"]["__TOutput__"]} start
     * @returns {Promise<OperationHistoryObject[]>}
     */

  }, {
    key: "getRelativeContractHistory",
    value: function getRelativeContractHistory(contract, stop, limit, start) {
      return this.exec('get_relative_contract_history', [contract, stop, limit, start]);
    }
    /**
     * @param {String} address
     * @param {Integer_t["uint64"]["__TOutput__"]} stop
     * @param {Integer_t["uint32"]["__TOutput__"]} limit
     * @param {Integer_t["uint64"]["__TOutput__"]} start
     * @returns {Promise}
     */

  }, {
    key: "getAccountAddressHistory",
    value: function getAccountAddressHistory(address, start, stop, limit) {
      return this.exec('get_account_address_history', [address, start, stop, limit]);
    }
  }]);
  return HistoryAPI;
}(_baseApi["default"]);

var _default = HistoryAPI;
exports["default"] = _default;