"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.echo = exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs3/regenerator"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/toConsumableArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _cache = _interopRequireDefault(require("./cache"));

var _api = _interopRequireDefault(require("./api"));

var _subscriber = _interopRequireDefault(require("./subscriber"));

var _transaction = _interopRequireDefault(require("./transaction"));

var _wsConstants = require("../constants/ws-constants");

var _apis = require("./apis");

var _engine = _interopRequireDefault(require("./engine"));

var _providers = require("./providers");

var _validators = require("../utils/validators");

/* global window */

/** @typedef {{ batch?: number, timeout?: number }} RegistrationOptions */

/** @typedef {{ cache?: import("./cache").Options, apis?: string[], registration?: RegistrationOptions }} Options */
var Echo = /*#__PURE__*/function () {
  function Echo() {
    var _context;

    (0, _classCallCheck2["default"])(this, Echo);

    /** @type {EchoApiEngine | null} */
    this.engine = null;
    this._isInitModules = false;
    this.initEchoApi = (0, _bind["default"])(_context = this.initEchoApi).call(_context, this);
    this.walletApi = new _apis.WalletApi();
    this.subscriber = new _subscriber["default"]();
  }

  (0, _createClass2["default"])(Echo, [{
    key: "isConnected",
    get: function get() {
      if (this.engine === null) return false;
      if (this.engine.provider.connectionType === _providers.ConnectionType.HTTP) return true;
      return this.engine.provider.connected;
    }
    /**
     * @readonly
     * @type {Set<string>}
     */

  }, {
    key: "apis",
    get: function get() {
      return new _set["default"](this.engine.apis);
    }
    /**
     * @param {string} address
     * @param {Options} options
     * @private
     */

  }, {
    key: "_connectToNode",
    value: function () {
      var _connectToNode2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(address, options) {
        var optionError, provider, apis;
        return _regenerator["default"].wrap(function _callee$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if ((0, _validators.validateUrl)(address)) {
                  _context2.next = 2;
                  break;
                }

                throw new Error("Invalid address ".concat(address));

              case 2:
                if (!(typeof window !== 'undefined' && window.location && window.location.protocol === 'https:' && !(0, _startsWith["default"])(address).call(address, 'wss://') && !(0, _startsWith["default"])(address).call(address, 'https://'))) {
                  _context2.next = 4;
                  break;
                }

                throw new Error('Secure domains require wss/https connection');

              case 4:
                optionError = (0, _validators.validateOptionsError)(options);

                if (!optionError) {
                  _context2.next = 7;
                  break;
                }

                throw new Error(optionError);

              case 7:
                provider = (0, _startsWith["default"])(address).call(address, 'ws') ? new _providers.WsProvider() : new _providers.HttpProvider(address);

                if (!(provider.connectionType === _providers.ConnectionType.WS)) {
                  _context2.next = 11;
                  break;
                }

                _context2.next = 11;
                return provider.connect(address, options);

              case 11:
                apis = options.apis || (provider.connectionType === _providers.ConnectionType.WS ? _wsConstants.DEFAULT_CHAIN_APIS : [_wsConstants.CHAIN_API.DATABASE_API]);
                this.engine = new _engine["default"](apis, provider);

                if (!this._isInitModules) {
                  _context2.next = 15;
                  break;
                }

                return _context2.abrupt("return");

              case 15:
                this.subscriber.setOptions(options);
                _context2.next = 18;
                return this._initModules(options);

              case 18:
                if (!options.store && this.store) {
                  options.store = this.store;
                }

                this.cache.setOptions(options);

              case 20:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee, this);
      }));

      function _connectToNode(_x, _x2) {
        return _connectToNode2.apply(this, arguments);
      }

      return _connectToNode;
    }()
    /**
     * @param {string} address
     * @param {Options} options
     */

  }, {
    key: "connect",
    value: function () {
      var _connect = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(address) {
        var _context3;

        var options,
            _args2 = arguments;
        return _regenerator["default"].wrap(function _callee2$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                options = _args2.length > 1 && _args2[1] !== undefined ? _args2[1] : {};

                if (!this.isConnected) {
                  _context4.next = 3;
                  break;
                }

                throw new Error('Connected');

              case 3:
                _context4.next = 5;
                return _promise["default"].all((0, _concat["default"])(_context3 = []).call(_context3, (0, _toConsumableArray2["default"])(address ? [this._connectToNode(address, options)] : []), (0, _toConsumableArray2["default"])(options.wallet ? [this.walletApi.connect(options.wallet, options)] : [])));

              case 5:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee2, this);
      }));

      function connect(_x3) {
        return _connect.apply(this, arguments);
      }

      return connect;
    }()
    /** @param {Options} options */

  }, {
    key: "_initModules",
    value: function () {
      var _initModules2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(options) {
        return _regenerator["default"].wrap(function _callee3$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                this._isInitModules = true;
                this.cache = new _cache["default"](options.cache);
                this.api = new _api["default"](this.cache, this.engine, options.registration);
                _context5.next = 5;
                return this.initEchoApi();

              case 5:
                if (this.engine.provider.connectionType === _providers.ConnectionType.WS) {
                  this.engine.provider.on(_wsConstants.STATUS.OPEN, this.initEchoApi);
                }

              case 6:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee3, this);
      }));

      function _initModules(_x4) {
        return _initModules2.apply(this, arguments);
      }

      return _initModules;
    }()
  }, {
    key: "initEchoApi",
    value: function () {
      var _initEchoApi = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4() {
        return _regenerator["default"].wrap(function _callee4$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return this.engine.init();

              case 2:
                if (!(this.engine.provider.connectionType === _providers.ConnectionType.WS)) {
                  _context6.next = 5;
                  break;
                }

                _context6.next = 5;
                return this.subscriber.init(this.cache, this.engine, this.api);

              case 5:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee4, this);
      }));

      function initEchoApi() {
        return _initEchoApi.apply(this, arguments);
      }

      return initEchoApi;
    }()
  }, {
    key: "syncCacheWithStore",
    value: function syncCacheWithStore(store) {
      if (this.isConnected) {
        this.cache.setStore({
          store: store
        });
      }

      this.store = store;
    }
  }, {
    key: "reconnect",
    value: function () {
      var _reconnect = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5() {
        return _regenerator["default"].wrap(function _callee5$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                if (!(this.engine === null)) {
                  _context7.next = 2;
                  break;
                }

                throw new Error('Not connected');

              case 2:
                if (!(this.engine.provider.connectionType === _providers.ConnectionType.HTTP)) {
                  _context7.next = 4;
                  break;
                }

                return _context7.abrupt("return");

              case 4:
                this.engine.provider.removeListener(_wsConstants.STATUS.OPEN, this.initEchoApi);
                _context7.next = 7;
                return this.engine.provider.reconnect();

              case 7:
                _context7.next = 9;
                return this.initEchoApi();

              case 9:
                this.engine.provider.on(_wsConstants.STATUS.OPEN, this.initEchoApi);

              case 10:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee5, this);
      }));

      function reconnect() {
        return _reconnect.apply(this, arguments);
      }

      return reconnect;
    }()
  }, {
    key: "disconnect",
    value: function () {
      var _disconnect = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6() {
        return _regenerator["default"].wrap(function _callee6$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                if (!(this.engine === null)) {
                  _context8.next = 2;
                  break;
                }

                throw new Error('Not connected');

              case 2:
                if (this.engine.provider.connectionType === _providers.ConnectionType.WS) {
                  this.subscriber.callCbOnDisconnect();
                  this.subscriber.reset();
                }

                this.cache.reset();

                if (!(this.engine.provider.connectionType === _providers.ConnectionType.WS)) {
                  _context8.next = 7;
                  break;
                }

                _context8.next = 7;
                return this.engine.provider.close();

              case 7:
                this.onOpen = null;
                this._isInitModules = false;

                if (this.engine.provider.connectionType === _providers.ConnectionType.WS) {
                  this.engine.provider.removeListener(_wsConstants.STATUS.OPEN, this.initEchoApi);
                }

              case 10:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee6, this);
      }));

      function disconnect() {
        return _disconnect.apply(this, arguments);
      }

      return disconnect;
    }()
    /** @returns {Transaction} */

  }, {
    key: "createTransaction",
    value: function createTransaction() {
      return new _transaction["default"](this.api);
    }
  }]);
  return Echo;
}();

var _default = Echo;
exports["default"] = _default;
var echo = new Echo();
exports.echo = echo;