"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

_Object$defineProperty(exports, "MapSerializer", {
  enumerable: true,
  get: function get() {
    return _Map["default"];
  }
});

_Object$defineProperty(exports, "OptionalSerializer", {
  enumerable: true,
  get: function get() {
    return _Optional["default"];
  }
});

_Object$defineProperty(exports, "SetSerializer", {
  enumerable: true,
  get: function get() {
    return _Set["default"];
  }
});

_Object$defineProperty(exports, "StaticVariantSerializer", {
  enumerable: true,
  get: function get() {
    return _StaticVariant["default"];
  }
});

_Object$defineProperty(exports, "StructSerializer", {
  enumerable: true,
  get: function get() {
    return _Struct["default"];
  }
});

_Object$defineProperty(exports, "VectorSerializer", {
  enumerable: true,
  get: function get() {
    return _Vector["default"];
  }
});

exports.vector = exports.structWithVariantKeys = exports.struct = exports.staticVariant = exports.set = exports.pair = exports.optional = exports.map = void 0;

var _Map = _interopRequireDefault(require("./Map"));

var _Optional = _interopRequireDefault(require("./Optional"));

var _Set = _interopRequireDefault(require("./Set"));

var _StaticVariant = _interopRequireDefault(require("./StaticVariant"));

var _Struct = _interopRequireDefault(require("./Struct"));

var _Vector = _interopRequireDefault(require("./Vector"));

var _Pair = _interopRequireDefault(require("./Pair"));

var _StructWithVariantKeys = _interopRequireDefault(require("./StructWithVariantKeys"));

/** @typedef {import("../ISerializer").default} ISerializer */

/**
 * @template {ISerializer} TKey
 * @template {ISerializer} TValue
 * @param {TKey} keySerializer
 * @param {TValue} valueSerializer
 * @returns {MapSerializer<TKey, TValue>}
 */
var map = function map(keySerializer, valueSerializer) {
  return new _Map["default"](keySerializer, valueSerializer);
};
/**
 * @template {ISerializer} TKey
 * @template {ISerializer} TValue
 * @param {TKey} firstSerializer
 * @param {TValue} secondSerializer
 * @returns {MapSerializer<TKey, TValue>}
 */


exports.map = map;

var pair = function pair(firstSerializer, secondSerializer) {
  return new _Pair["default"](firstSerializer, secondSerializer);
};
/**
 * @template {ISerializer} T
 * @param {T} serializer
 * @returns {OptionalSerializer<T>}
 */


exports.pair = pair;

var optional = function optional(serializer) {
  return new _Optional["default"](serializer);
};
/**
 * @template {ISerializer} T
 * @param {T} serializer
 * @returns {SetSerializer<T>}
 */


exports.optional = optional;

var set = function set(serializer) {
  return new _Set["default"](serializer);
};
/** @typedef {import("./StaticVariant").Variants} Variants */

/** @typedef {import("./Struct").SerializersMap} SerializersMap */

/**
 * @template {Variants} T
 * @param {T} serializers
 * @returns {StaticVariantSerializer<T>}
 */


exports.set = set;

var staticVariant = function staticVariant(serializers) {
  return new _StaticVariant["default"](serializers);
};
/**
 * @template {SerializersMap} T
 * @param {T} serializers
 * @returns {StructSerializer<T>}
 */


exports.staticVariant = staticVariant;

var struct = function struct(serializers) {
  return new _Struct["default"](serializers);
};
/**
 * @template {SerializersMap} T
 * @template { {
 * 	keyIndexInStructure: number,
 * 	serializersData: [string, ISerializer][],
 * }[] } V
 * @param {T} serializers
 * @param {V} extraSerializers
 * @returns {StructWithVariantKeysSerializer<T, V>}
 */


exports.struct = struct;

var structWithVariantKeys = function structWithVariantKeys(serializers, extraSerializers) {
  return new _StructWithVariantKeys["default"](serializers, extraSerializers);
};
/**
 * @template {ISerializer} T
 * @param {T} serializer
 * @returns {VectorSerializer<T>}
 */


exports.structWithVariantKeys = structWithVariantKeys;

var vector = function vector(serializer) {
  return new _Vector["default"](serializer);
};

exports.vector = vector;