"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.BigNumberToLong = BigNumberToLong;
exports.toId = toId;
exports.toRawContractLogsFilterOptions = toRawContractLogsFilterOptions;

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _parseInt = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/number/parse-int"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/slicedToArray"));

var _bytebuffer = require("bytebuffer");

var _bignumber = _interopRequireDefault(require("bignumber.js"));

var _validators = require("./validators");

var _serializers = require("../serializers");

var MAX_UINT64 = new _bignumber["default"](2).pow(64).minus(1);
var MAX_INT64 = new _bignumber["default"](2).pow(63).minus(1);
/**
 * @param {BigNumber} bn
 * @param {boolean} unsigned
 */

function BigNumberToLong(bn, unsigned) {
  if (!_bignumber["default"].isBigNumber(bn)) throw new Error('value is not a BigNumber');
  if (typeof unsigned !== 'boolean') throw new Error('property "unsigned" is not a boolean');
  if (!bn.isInteger()) throw new Error('value is not a integer');

  if (unsigned) {
    if (bn.gt(MAX_UINT64)) throw new Error('uint64 overflow');
    if (bn.lt(0)) throw new Error('value is negative');
  } else if (bn.abs().gt(MAX_INT64)) throw new Error('int64 overflow');

  return _bytebuffer.Long.fromString(bn.toString(10), unsigned, 10);
}
/**
 * @param {string|number} address
 * @param {number} reservedSpaceId
 * @param {number|Array<number>} objectTypeId
 * @returns {number}
 */


function toId(address, reservedSpaceId, objectTypeId) {
  var _context;

  if (typeof address === 'number') return address;
  if (typeof address !== 'string') throw new Error('invalid id type');
  if (!_validators.idRegex.test(address)) throw new Error('invalid id format'); // TODO: use BigNumber for id

  var _address$split$map = (0, _map["default"])(_context = address.split('.')).call(_context, function (str) {
    return (0, _parseInt["default"])(str, 10);
  }),
      _address$split$map2 = (0, _slicedToArray2["default"])(_address$split$map, 3),
      actualReservedSpaceId = _address$split$map2[0],
      actualObjectTypeId = _address$split$map2[1],
      id = _address$split$map2[2];

  if (actualReservedSpaceId !== reservedSpaceId) throw new Error('invalid reservedSpaceId');

  if (typeof objectTypeId === 'number') {
    if (actualObjectTypeId !== objectTypeId) throw new Error('invalid objectTypeId');
  } else if (!(0, _isArray["default"])(objectTypeId)) throw new Error('invalid objectTypeId type');else if (!(0, _includes["default"])(objectTypeId).call(objectTypeId, actualObjectTypeId)) throw new Error('invalid objectTypeId');

  return id;
}
/** @typedef {string | number | BigNumber} ObjectId_t */

/** @typedef {number | BigNumber | string} Integer_t */

/**
 * @template T
 * @typedef {Set<T> | T[] | undefined} Set_t
 */

/**
 * @typedef {Object} ContractLogsFilterOptions_t
 * @property {Set_t<ObjectId_t>} [contracts]
 * @property {Array<Set_t<string>>} [topics]
 * @property {Integer_t} [fromBlock]
 * @property {Integer_t} [toBlock]
 */

/**
 * @typedef {Object} ContractLogsFilterOptionsRaw
 * @property {string[]} [contracts]
 * @property {Array<string[]>} [topics]
 * @property {number} [from_block]
 * @property {number} [to_block]
 */

/**
 * @param {ContractLogsFilterOptions_t} opts
 * @returns {ContractLogsFilterOptionsRaw}
 */


function toRawContractLogsFilterOptions(opts) {
  /** @type {string[]} */
  var contractsList = opts.contracts === undefined ? undefined : _serializers.collections.set(_serializers.chain.ids.protocol.contractId).toRaw(opts.contracts, 'contracts');
  /** @type {Array<string[]>} */

  var topicsList = opts.topics === undefined ? undefined : _serializers.collections.vector(_serializers.collections.set(_serializers.basic.string)).toRaw(opts.topics, '`topics`');
  var from = opts.fromBlock === undefined ? undefined : _serializers.basic.integers.int32.toRaw(opts.fromBlock, 'fromBlock');
  var to = opts.toBlock === undefined ? undefined : _serializers.basic.integers.int32.toRaw(opts.toBlock, 'toBlock');
  if (from !== undefined && from < 0) throw new Error('`fromBlock` must be greater than or equal to zero');
  if (to !== undefined && to <= 0) throw new Error('`toBlock` must be greater than zero');
  return {
    contracts: contractsList,
    topics: topicsList,
    from_block: from,
    to_block: to
  };
}