"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.solveRegistrationTask = void 0;
exports.validateRegistrationOptions = validateRegistrationOptions;

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs3/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/asyncToGenerator"));

var _isSafeInteger = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/number/is-safe-integer"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _setTimeout2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set-timeout"));

var _assert = require("assert");

var _hash = require("../crypto/hash");

var _integers = require("../serializers/basic/integers");

/** @typedef {import("../echo").RegistrationOptions} RegistrationOptions */
var getHashPow = function getHashPow(hash) {
  var solPow = 0;

  for (var index = 0; index < 32; index += 1) {
    var _byte = hash[index];

    if (_byte === 0) {
      solPow += 8;
      continue;
    }
    /* eslint-disable no-bitwise */


    var divider = 1 << 7;

    while (_byte < divider) {
      solPow += 1;
      /* eslint-disable no-bitwise */

      divider >>= 1;
    }

    break;
  }

  return solPow;
};
/**
 * @param {RegistrationOptions} options
 * @returns {Required<RegistrationOptions>}
 */


function validateRegistrationOptions(options) {
  if (options.batch !== undefined) {
    (0, _assert.ok)(typeof options.batch === 'number');
    (0, _assert.ok)((0, _isSafeInteger["default"])(options.batch));
    (0, _assert.ok)(options.batch >= 0);
  }

  if (options.timeout) {
    (0, _assert.ok)(typeof options.timeout === 'number');
    (0, _assert.ok)((0, _isSafeInteger["default"])(options.timeout));
    (0, _assert.ok)(options.timeout >= 0);
  }

  var batch = options.batch === undefined ? 1e6 : options.batch;
  var timeout = options.timeout === undefined ? 100 : options.timeout;
  return {
    batch: batch,
    timeout: timeout
  };
}
/**
 *
 * @param {String} blockId
 * @param {String} randNum
 * @param {Number} difficulty
 * @param {RegistrationOptions} [options]
 */


var solveRegistrationTask = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(blockId, randNum, difficulty) {
    var options,
        _validateRegistration,
        batch,
        timeout,
        buffer,
        nonce,
        bufferToHash,
        hash,
        hashPow,
        _args = arguments;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            options = _args.length > 3 && _args[3] !== undefined ? _args[3] : {};
            _validateRegistration = validateRegistrationOptions(options), batch = _validateRegistration.batch, timeout = _validateRegistration.timeout;
            buffer = (0, _concat["default"])(Buffer).call(Buffer, [Buffer.from(blockId, 'hex'), _integers.uint64.serialize(randNum)]);
            nonce = 0;

          case 4:
            if (!true) {
              _context.next = 16;
              break;
            }

            bufferToHash = (0, _concat["default"])(Buffer).call(Buffer, [buffer, _integers.uint64.serialize(nonce)]);
            hash = (0, _hash.sha256)(bufferToHash);
            hashPow = getHashPow(hash); // eslint-disable-next-line no-await-in-loop

            if (!(batch !== 0 && nonce % batch === 0)) {
              _context.next = 11;
              break;
            }

            _context.next = 11;
            return new _promise["default"](function (resolve) {
              return (0, _setTimeout2["default"])(function () {
                return resolve();
              }, timeout);
            });

          case 11:
            if (!(hashPow > difficulty)) {
              _context.next = 13;
              break;
            }

            return _context.abrupt("return", nonce);

          case 13:
            nonce += 1;
            _context.next = 4;
            break;

          case 16:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function solveRegistrationTask(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

exports.solveRegistrationTask = solveRegistrationTask;